import 'package:flutter/material.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:factory_signage_app/const/colors.dart';
import 'package:factory_signage_app/const/dimens.dart';
import 'package:factory_signage_app/const/string.dart';
import 'package:factory_signage_app/provider.dart';

class Schedule extends StatelessWidget {
  const Schedule(
      {super.key, this.size = const Size(double.infinity, double.infinity)});

  final Size size;

  @override
  Widget build(BuildContext context) {
    final Size widgetSize = MediaQuery.of(context).size;

    return SizedBox(
      width: size.width,
      height: size.height,
      child: Column(
        children: [
          title(widgetSize),
          Expanded(
            child: timeline(widgetSize),
          ),
        ],
      ),
    );
  }

  Widget title(Size widgetSize) {
    return Container(
      alignment: Alignment.center,
      color: textBackgroundColor,
      width: double.infinity,
      height: scheduleTitleHeightRatio * widgetSize.height,
      child: Text(
        scheduleTitle,
        style: TextStyle(
          color: textWhiteColor,
          fontSize: scheduleTitleFontSizeRatio * widgetSize.height,
        ),
      ),
    );
  }

  Widget timeline(Size widgetSize) {
    return Table(
      border: TableBorder.all(),
      columnWidths: <int, TableColumnWidth>{
        0: FixedColumnWidth(scheduleTimeWidthRatio * widgetSize.width),
        1: const FlexColumnWidth(),
      },
      children: [...scheduleTableRows(widgetSize)],
    );
  }

  List<TableRow> scheduleTableRows(Size widgetSize) {
    final List<TableRow> tableRows = [];
    for (int i = 0; i < scheduleProgresses.length; i++) {
      final tableRow = TableRow(
        children: [
          // Only A Widget is subject to update.
          Consumer(
            builder: (context, ref, _) => scheduleTime(ref, widgetSize, i),
          ),
          Column(
            children: [
              scheduleProgress(widgetSize, i),
              scheduleWorker(widgetSize, i),
            ],
          ),
        ],
      );
      tableRows.add(tableRow);
    }
    return tableRows;
  }

  Widget scheduleTime(WidgetRef ref, Size widgetSize, int num) {
    bool working = isWorking(
      ref,
      scheduleStartTimes[num],
      scheduleEndTimes[num],
    );
    return Container(
      alignment: Alignment.center,
      color: working
          ? workingBackgroundColor
          : scheduleBackgroundColor,
      height: scheduleTimeHeightRatio * widgetSize.height,
      child: Text(
        '${scheduleStartTimes[num]}\n|\n${scheduleEndTimes[num]}',
        style: TextStyle(
          color: working ? textWhiteColor : textBlackColor,
          fontSize: scheduleTimeFontSizeRatio * widgetSize.height,
          fontWeight: working ? FontWeight.bold : FontWeight.normal,
          height: scheduleTimeStyleHeight,
        ),
        textAlign: TextAlign.center,
      ),
    );
  }

  Widget scheduleProgress(Size widgetSize, int num) {
    return Container(
      // When text is aligned in alignment, it is misaligned depending on the length of the text.
      // Use spaces to move the text slightly away from the left.
      alignment: Alignment.centerLeft,
      color: scheduleBackgroundColor,
      width: double.infinity,
      height: scheduleProgressHeightRatio * widgetSize.height,
      child: Text(
        '  ${scheduleProgresses[num]}',
        style: TextStyle(
          color: textBlackColor,
          fontSize: scheduleProgressFontSizeRatio * widgetSize.height,
          height: scheduleTimeStyleHeight,
        ),
      ),
    );
  }

  Widget scheduleWorker(Size widgetSize, int num) {
    return Container(
      alignment: Alignment.centerLeft,
      color: workerTextBackgroundColor,
      width: double.infinity,
      height: scheduleWorkerHeightRatio * widgetSize.height,
      child: Text(
        '  ${scheduleWorkers[num]}',
        style: TextStyle(
          color: textWorkerColor,
          fontSize: scheduleWorkerFontSizeRatio * widgetSize.height,
          height: scheduleTimeStyleHeight,
        ),
      ),
    );
  }

  bool isWorking(WidgetRef ref, String startTime, String endTime) {
    final presentDate = ref.watch(presentDateTimeProvider);
    // Set to January 1, 1900 to compare hours and minutes of the current time
    final compareDate = DateTime(
        1900, 1, 1, presentDate.hour, presentDate.minute, presentDate.second);
    final startDate = stringToDateTime(startTime);
    final endDate = stringToDateTime(endTime);
    return compareDate.isAfter(startDate) && compareDate.isBefore(endDate);
  }

  DateTime stringToDateTime(String time) {
    // If the time is a human digit, add a leading zero.
    if (time.length == 4) {
      time = '0$time';
    }
    return DateTime.parse('1900-01-01 $time:00');
  }
}
